@@ -1,5 +1,5 @@
 /*  Lziprecover - Data recovery tool for the lzip format
-    Copyright (C) 2009-2018 Antonio Diaz Diaz.
+    Copyright (C) 2009-2019 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@ public:
     :
     buffer( buf ),
     buffer_size( buf_size ),
-    pos( File_header::size ),
+    pos( Lzip_header::size ),
     code( 0 ),
     range( 0xFFFFFFFFU ),
     at_stream_end( false )
@@ -47,11 +47,11 @@ public:
     return buffer[pos++];
     }
 
-  const File_trailer * get_trailer()
+  const Lzip_trailer * get_trailer()
     {
-    if( buffer_size - pos < File_trailer::size ) return 0;
-    const File_trailer * const p = (File_trailer *)( buffer + pos );
-    pos += File_trailer::size;
+    if( buffer_size - pos < Lzip_trailer::size ) return 0;
+    const Lzip_trailer * const p = (const Lzip_trailer *)( buffer + pos );
+    pos += Lzip_trailer::size;
     return p;
     }
 
@@ -69,24 +69,23 @@ public:
       { range <<= 8; code = (code << 8) | get_byte(); }
     }
 
-  int decode( const int num_bits )
+  unsigned decode( const int num_bits )
     {
-    int symbol = 0;
+    unsigned symbol = 0;
     for( int i = num_bits; i > 0; --i )
       {
       normalize();
       range >>= 1;
 //      symbol <<= 1;
 //      if( code >= range ) { code -= range; symbol |= 1; }
-      const uint32_t mask = 0U - (code < range);
-      code -= range;
-      code += range & mask;
-      symbol = (symbol << 1) + (mask + 1);
+      const bool bit = ( code >= range );
+      symbol = ( symbol << 1 ) + bit;
+      code -= range & ( 0U - bit );
       }
     return symbol;
     }
 
-  int decode_bit( Bit_model & bm )
+  unsigned decode_bit( Bit_model & bm )
     {
     normalize();
     const uint32_t bound = ( range >> bit_model_total_bits ) * bm.probability;
@@ -105,18 +104,18 @@ public:
       }
     }
 
-  int decode_tree3( Bit_model bm[] )
+  unsigned decode_tree3( Bit_model bm[] )
     {
-    int symbol = 1;
+    unsigned symbol = 1;
     symbol = ( symbol << 1 ) | decode_bit( bm[symbol] );
     symbol = ( symbol << 1 ) | decode_bit( bm[symbol] );
     symbol = ( symbol << 1 ) | decode_bit( bm[symbol] );
     return symbol & 7;
     }
 
-  int decode_tree6( Bit_model bm[] )
+  unsigned decode_tree6( Bit_model bm[] )
     {
-    int symbol = 1;
+    unsigned symbol = 1;
     symbol = ( symbol << 1 ) | decode_bit( bm[symbol] );
     symbol = ( symbol << 1 ) | decode_bit( bm[symbol] );
     symbol = ( symbol << 1 ) | decode_bit( bm[symbol] );
@@ -126,49 +125,47 @@ public:
     return symbol & 0x3F;
     }
 
-  int decode_tree8( Bit_model bm[] )
+  unsigned decode_tree8( Bit_model bm[] )
     {
-    int symbol = 1;
-    while( symbol < 0x100 )
+    unsigned symbol = 1;
+    for( int i = 0; i < 8; ++i )
       symbol = ( symbol << 1 ) | decode_bit( bm[symbol] );
     return symbol & 0xFF;
     }
 
-  int decode_tree_reversed( Bit_model bm[], const int num_bits )
+  unsigned decode_tree_reversed( Bit_model bm[], const int num_bits )
     {
-    int model = 1;
-    int symbol = 0;
+    unsigned model = 1;
+    unsigned symbol = 0;
     for( int i = 0; i < num_bits; ++i )
       {
-      const bool bit = decode_bit( bm[model] );
-      model <<= 1;
-      if( bit ) { ++model; symbol |= (1 << i); }
+      const unsigned bit = decode_bit( bm[model] );
+      model = ( model << 1 ) + bit;
+      symbol |= ( bit << i );
       }
     return symbol;
     }
 
-  int decode_tree_reversed4( Bit_model bm[] )
+  unsigned decode_tree_reversed4( Bit_model bm[] )
     {
-    int model = 1;
-    int symbol = decode_bit( bm[model] );
-    model = (model << 1) + symbol;
-    int bit = decode_bit( bm[model] );
-    model = (model << 1) + bit; symbol |= (bit << 1);
+    unsigned symbol = decode_bit( bm[1] );
+    unsigned model = 2 + symbol;
+    unsigned bit = decode_bit( bm[model] );
+    model = ( model << 1 ) + bit; symbol |= ( bit << 1 );
     bit = decode_bit( bm[model] );
-    model = (model << 1) + bit; symbol |= (bit << 2);
-    if( decode_bit( bm[model] ) ) symbol |= 8;
+    model = ( model << 1 ) + bit; symbol |= ( bit << 2 );
+    symbol |= ( decode_bit( bm[model] ) << 3 );
     return symbol;
     }
 
-  int decode_matched( Bit_model bm[], int match_byte )
+  unsigned decode_matched( Bit_model bm[], unsigned match_byte )
     {
     Bit_model * const bm1 = bm + 0x100;
-    int symbol = 1;
+    unsigned symbol = 1;
     while( symbol < 0x100 )
       {
-      match_byte <<= 1;
-      const int match_bit = match_byte & 0x100;
-      const int bit = decode_bit( bm1[match_bit+symbol] );
+      const unsigned match_bit = ( match_byte <<= 1 ) & 0x100;
+      const unsigned bit = decode_bit( bm1[match_bit+symbol] );
       symbol = ( symbol << 1 ) | bit;
       if( match_bit != bit << 8 )
         {
@@ -180,7 +177,7 @@ public:
     return symbol & 0xFF;
     }
 
-  int decode_len( Len_model & lm, const int pos_state )
+  unsigned decode_len( Len_model & lm, const int pos_state )
     {
     if( decode_bit( lm.choice1 ) == 0 )
       return decode_tree3( lm.bm_low[pos_state] );
@@ -223,20 +220,17 @@ class LZ_mtester
   Len_model match_len_model;
   Len_model rep_len_model;
 
+  void print_block( const int len );
   void flush_data();
   bool verify_trailer();
-  void print_block( const int len );
 
   uint8_t peek_prev() const
-    {
-    const unsigned i = ( ( pos > 0 ) ? pos : dictionary_size ) - 1;
-    return buffer[i];
-    }
+    { return buffer[((pos > 0) ? pos : dictionary_size)-1]; }
 
   uint8_t peek( const unsigned distance ) const
     {
-    unsigned i = pos - distance - 1;
-    if( pos <= distance ) i += dictionary_size;
+    const unsigned i = ( ( pos > distance ) ? 0 : dictionary_size ) +
+                       pos - distance - 1;
     return buffer[i];
     }
 
@@ -248,17 +242,26 @@ class LZ_mtester
 
   void copy_block( const unsigned distance, unsigned len )
     {
-    unsigned i = pos - distance - 1;
-    bool fast;
-    if( pos <= distance )
-      { i += dictionary_size;
-        fast = ( len <= dictionary_size - i && len <= i - pos ); }
+    unsigned lpos = pos, i = lpos - distance - 1;
+    bool fast, fast2;
+    if( lpos > distance )
+      {
+      fast = ( len < dictionary_size - lpos );
+      fast2 = ( fast && len <= lpos - i );
+      }
     else
-      fast = ( len < dictionary_size - pos && len <= pos - i );
-    if( fast )					// no wrap, no overlap
+      {
+      i += dictionary_size;
+      fast = ( len < dictionary_size - i );	// (i == pos) may happen
+      fast2 = ( fast && len <= i - lpos );
+      }
+    if( fast )					// no wrap
       {
-      std::memcpy( buffer + pos, buffer + i, len );
       pos += len;
+      if( fast2 )				// no wrap, no overlap
+        std::memcpy( buffer + lpos, buffer + i, len );
+      else
+        for( ; len > 0; --len ) buffer[lpos++] = buffer[i++];
       }
     else for( ; len > 0; --len )
       {
@@ -288,7 +291,8 @@ public:
     rep3( 0 ),
     max_rep0( 0 ),
     pos_wrapped( false )
-    { buffer[dictionary_size-1] = 0; }		// prev_byte of first byte
+    // prev_byte of first byte; also for peek( 0 ) on corrupt file
+    { buffer[dictionary_size-1] = 0; }
 
   ~LZ_mtester() { delete[] buffer; }
 
@@ -303,12 +307,3 @@ public:
   int debug_decode_member( const long long dpos, const long long mpos,
                            const bool show_packets );	// sets max_rep0
   };
-
-
-uint8_t * read_member( const int infd, const long long mpos,
-                       const long long msize );
-const LZ_mtester * prepare_master( const uint8_t * const buffer,
-                                   const long buffer_size,
-                                   const unsigned long pos_limit,
-                                   const unsigned dictionary_size );
-bool test_member_rest( const LZ_mtester & master, long * const failure_posp = 0 );
